{% extends 'base.j2' %}
{% block title %}settings{% endblock %}
{% block content %}
<h1>Settings</h1>
<p>post_min_replies: {{ post_min_replies }}</p>
<p>viral_reply_min_replies: {{ viral_reply_min_replies }}</p>
<h2>Exclude Boards</h2>
{% endblock %}
