
.text-col {
    /* width: 1000px; */
    width: 800px;
}

table {
    white-space: nowrap;
    margin: auto;
    background-color: rgba(0, 0, 0, 0.03);
    border-spacing: 30px 0;
}

th {
    font-family: Verdana;
    font-size: 9pt;
    font-weight: normal;
    text-align: left;
    max-width: 40vw;
    text-overflow: ellipsis;
    overflow: hidden;
}

a { 
    color: var(--text-main);
    text-decoration: none; 
}

a:visited {
    color: rgb(88, 88, 88);
}

@media (prefers-color-scheme: light) {
    a:visited {
        color: rgb(200, 200, 200);
    }
}
