<html>
<head>
    <link rel="stylesheet" href="https://cdn.jsdelivr.net/npm/water.css@2/out/water.min.css"/>
    <link rel="stylesheet" href="/static/css/base.css"/>
    {% block extra_css %}{% endblock %}
    {% block extra_js %}{% endblock %}
    <title>{% block title %}{% endblock %}</title>
</head>
<body>
    <nav>
        <span id="nav-left">
            <a href="/">home</a>
            <a href="/stream">stream</a>
        </span>
        
        <span id="nav-right">
            <a href="/settings">settings</a>
            <img align='center' src='https://s2.googleusercontent.com/s2/favicons?domain=github.com'/>
            <a href="https://github.com/tandav/chans">github</a>
        </span>
    </nav>
    <section>
    {% block content %}
    {% endblock %}
    </section>
</body>
</html>
